@@ -37,6 +37,7 @@
 #include "ast/transform/IOAttributes.h"
 #include "ast/transform/IODefaults.h"
 #include "ast/transform/InlineRelations.h"
+#include "ast/transform/InsertLatticeOperations.h"
 #include "ast/transform/MagicSet.h"
 #include "ast/transform/MaterializeAggregationQueries.h"
 #include "ast/transform/MaterializeSingletonAggregation.h"
@@ -479,6 +480,7 @@ Own<ast::transform::PipelineTransformer> astTransformationPipeline(Global& glb)
     // Main pipeline
     auto pipeline = mk<ast::transform::PipelineTransformer>(mk<ast::transform::ComponentChecker>(),
             mk<ast::transform::ComponentInstantiationTransformer>(),
+            mk<ast::transform::LatticeTransformer>(),
             mk<ast::transform::DebugDeltaRelationTransformer>(),
             mk<ast::transform::IODefaultsTransformer>(),
             mk<ast::transform::SimplifyAggregateTargetExpressionTransformer>(),
@@ -597,6 +599,10 @@ const char* packageVersion() {
     return PACKAGE_VERSION;
 }
 
+const char* gitPackageVersion() {
+    return GIT_PACKAGE_VERSION;
+}
+
 std::size_t ramDomainSizeInBits() {
     return RAM_DOMAIN_SIZE;
 }
@@ -604,7 +610,11 @@ std::size_t ramDomainSizeInBits() {
 std::string versionFooter() {
     std::stringstream footer;
     footer << "----------------------------------------------------------------------------" << std::endl;
-    footer << "Version: " << packageVersion() << std::endl;
+    footer << "Version: " << packageVersion();
+    if (strlen(gitPackageVersion()) > 0) {
+        footer << " (" << gitPackageVersion() << ")";
+    }
+    footer << std::endl;
     footer << "Word size: " << ramDomainSizeInBits() << " bits" << std::endl;
     footer << "Options enabled:";
 #ifdef USE_LIBFFI
@@ -624,7 +634,7 @@ std::string versionFooter() {
 #endif
     footer << std::endl;
     footer << "----------------------------------------------------------------------------" << std::endl;
-    footer << "Copyright (c) 2016-22 The Souffle Developers." << std::endl;
+    footer << "Copyright (c) 2016-25 The Souffle Developers." << std::endl;
     footer << "Copyright (c) 2013-16 Oracle and/or its affiliates." << std::endl;
     footer << "All rights reserved." << std::endl;
     footer << "============================================================================" << std::endl;
@@ -786,24 +796,27 @@ int main(Global& glb, const char* souffle_executable) {
         }
 
         /* for the jobs option, to determine the number of threads used */
-#ifdef _OPENMP
         if (isNumber(glb.config().get("jobs").c_str())) {
-            if (std::stoi(glb.config().get("jobs")) < 1) {
+            int n = std::stoi(glb.config().get("jobs"));
+            if (n < 1) {
                 throw std::runtime_error("-j/--jobs may only be set to 'auto' or an integer greater than 0.");
             }
+#ifndef _OPENMP
+            if (n != 1 && !glb.config().has("no-warn")) {
+                std::cerr << "\nThis installation of Souffle does not support concurrent jobs.\n";
+            }
+#endif
         } else {
             if (!glb.config().has("jobs", "auto")) {
                 throw std::runtime_error("-j/--jobs may only be set to 'auto' or an integer greater than 0.");
             }
+#ifdef _OPENMP
             // set jobs to zero to indicate the synthesiser and interpreter to use the system default.
             glb.config().set("jobs", "0");
-        }
 #else
-        // Check that -j option has not been changed from the default
-        if (glb.config().get("jobs") != "1" && !glb.config().has("no-warn")) {
-            std::cerr << "\nThis installation of Souffle does not support concurrent jobs.\n";
-        }
+            glb.config().set("jobs", "1");
 #endif
+        }
 
         /* if an output directory is given, check it exists */
         if (glb.config().has("output-dir") && !glb.config().has("output-dir", "-") &&
