@@ -14,7 +14,7 @@
 namespace souffle::ast {
 
 AliasType::AliasType(QualifiedName name, QualifiedName aliasTypeName, SrcLocation loc)
-        : Type(std::move(name), std::move(loc)), aliasType(std::move(aliasTypeName)) {}
+        : Type(NK_AliasType, std::move(name), std::move(loc)), aliasType(std::move(aliasTypeName)) {}
 
 void AliasType::print(std::ostream& os) const {
     os << ".type " << getQualifiedName() << " = " << getAliasType();
@@ -29,4 +29,8 @@ AliasType* AliasType::cloning() const {
     return new AliasType(getQualifiedName(), getAliasType(), getSrcLoc());
 }
 
+bool AliasType::classof(const Node* n) {
+    return n->getKind() == NK_AliasType;
+}
+
 }  // namespace souffle::ast
