@@ -16,7 +16,7 @@
 namespace souffle::ast {
 
 BranchType::BranchType(QualifiedName name, VecOwn<Attribute> fields, SrcLocation loc)
-        : Node(std::move(loc)), name(std::move(name)), fields(std::move(fields)) {
+        : Node(NK_BranchType, std::move(loc)), name(std::move(name)), fields(std::move(fields)) {
     assert(allValidPtrs(this->fields));
 }
 
@@ -25,6 +25,7 @@ std::vector<Attribute*> BranchType::getFields() {
 }
 
 void BranchType::print(std::ostream& os) const {
+    printAnnotations(os);
     os << tfm::format("%s {%s}", name, join(fields, ", "));
 }
 
@@ -36,4 +37,12 @@ void BranchType::setFieldType(std::size_t idx, QualifiedName type) {
     fields.at(idx)->setTypeName(std::move(type));
 }
 
+bool BranchType::classof(const Node* n) {
+    return n->getKind() == NK_BranchType;
+}
+
+bool BranchType::equal(const Node& /* other */) const {
+    return false;
+}
+
 }  // namespace souffle::ast
