@@ -12,8 +12,10 @@
 #include <utility>
 
 namespace souffle::ast {
-Constant::Constant(std::string value, SrcLocation loc)
-        : Argument(std::move(loc)), constant(std::move(value)){};
+Constant::Constant(NodeKind kind, std::string value, SrcLocation loc)
+        : Argument(kind, std::move(loc)), constant(std::move(value)) {
+    assert(kind >= NK_Constant && kind < NK_LastConstant);
+};
 
 void Constant::print(std::ostream& os) const {
     os << getConstant();
@@ -24,4 +26,9 @@ bool Constant::equal(const Node& node) const {
     return constant == other.constant;
 }
 
+bool Constant::classof(const Node* n) {
+    const NodeKind kind = n->getKind();
+    return (kind >= NK_Constant && kind < NK_LastConstant);
+}
+
 }  // namespace souffle::ast
