@@ -11,6 +11,8 @@
 
 namespace souffle::ast {
 
+Counter::Counter(SrcLocation loc) : Argument(NK_Counter, std::move(loc)) {}
+
 void Counter::print(std::ostream& os) const {
     os << "$";
 }
@@ -19,4 +21,12 @@ Counter* Counter::cloning() const {
     return new Counter(getSrcLoc());
 }
 
+bool Counter::classof(const Node* n) {
+    return n->getKind() == NK_Counter;
+}
+
+bool Counter::equal(const Node&) const {
+    return true;
+}
+
 }  // namespace souffle::ast
