@@ -29,10 +29,16 @@ class Counter : public Argument {
 public:
     using Argument::Argument;
 
+    Counter(SrcLocation = {});
+
+    static bool classof(const Node*);
+
 protected:
     void print(std::ostream& os) const override;
 
 private:
+    bool equal(const Node&) const override;
+
     Counter* cloning() const override;
 };
 
