@@ -11,6 +11,8 @@
 
 namespace souffle::ast {
 
+IterationCounter::IterationCounter(SrcLocation loc) : Argument(NK_IterationCounter, loc) {}
+
 void IterationCounter::print(std::ostream& os) const {
     os << "recursive_iteration_cnt()";
 }
@@ -19,4 +21,12 @@ IterationCounter* IterationCounter::cloning() const {
     return new IterationCounter(getSrcLoc());
 }
 
+bool IterationCounter::classof(const Node* n) {
+    return n->getKind() == NK_IterationCounter;
+}
+
+bool IterationCounter::equal(const Node&) const {
+    return true;
+}
+
 }  // namespace souffle::ast
