@@ -27,12 +27,15 @@ namespace souffle::ast {
  */
 class IterationCounter : public Argument {
 public:
-    using Argument::Argument;
+    IterationCounter(SrcLocation loc = {});
+
+    static bool classof(const Node*);
 
 protected:
     void print(std::ostream& os) const override;
 
 private:
+    bool equal(const Node&) const override;
     IterationCounter* cloning() const override;
 };
 
