@@ -14,7 +14,8 @@
 
 namespace souffle::ast {
 
-Negation::Negation(Own<Atom> atom, SrcLocation loc) : Literal(std::move(loc)), atom(std::move(atom)) {
+Negation::Negation(Own<Atom> atom, SrcLocation loc)
+        : Literal(NK_Negation, std::move(loc)), atom(std::move(atom)) {
     assert(this->atom != nullptr);
 }
 
@@ -39,4 +40,8 @@ Negation* Negation::cloning() const {
     return new Negation(clone(atom), getSrcLoc());
 }
 
+bool Negation::classof(const Node* n) {
+    return n->getKind() == NK_Negation;
+}
+
 }  // namespace souffle::ast
