@@ -13,13 +13,14 @@
 
 namespace souffle::ast {
 
-NumericConstant::NumericConstant(RamSigned value) : Constant(std::to_string(value)), fixedType(Type::Int) {}
+NumericConstant::NumericConstant(RamSigned value)
+        : Constant(NK_NumericConstant, std::to_string(value)), fixedType(Type::Int) {}
 
 NumericConstant::NumericConstant(std::string constant, SrcLocation loc)
-        : Constant(std::move(constant), std::move(loc)) {}
+        : Constant(NK_NumericConstant, std::move(constant), std::move(loc)) {}
 
 NumericConstant::NumericConstant(std::string constant, std::optional<Type> fixedType, SrcLocation loc)
-        : Constant(std::move(constant), std::move(loc)), fixedType(fixedType) {}
+        : Constant(NK_NumericConstant, std::move(constant), std::move(loc)), fixedType(fixedType) {}
 
 bool NumericConstant::equal(const Node& node) const {
     const auto& other = asAssert<NumericConstant>(node);
@@ -30,4 +31,8 @@ NumericConstant* NumericConstant::cloning() const {
     return new NumericConstant(getConstant(), getFixedType(), getSrcLoc());
 }
 
+bool NumericConstant::classof(const Node* n) {
+    return n->getKind() == NK_NumericConstant;
+}
+
 }  // namespace souffle::ast
