@@ -14,9 +14,10 @@
 namespace souffle::ast {
 
 SubsetType::SubsetType(QualifiedName name, QualifiedName baseTypeName, SrcLocation loc)
-        : Type(std::move(name), std::move(loc)), baseType(std::move(baseTypeName)) {}
+        : Type(NK_SubsetType, std::move(name), std::move(loc)), baseType(std::move(baseTypeName)) {}
 
 void SubsetType::print(std::ostream& os) const {
+    printAnnotations(os);
     os << ".type " << getQualifiedName() << " <: " << getBaseType();
 }
 
@@ -29,4 +30,8 @@ SubsetType* SubsetType::cloning() const {
     return new SubsetType(getQualifiedName(), getBaseType(), getSrcLoc());
 }
 
+bool SubsetType::classof(const Node* n) {
+    return n->getKind() == NK_SubsetType;
+}
+
 }  // namespace souffle::ast
