@@ -11,6 +11,8 @@
 
 namespace souffle::ast {
 
+UnnamedVariable::UnnamedVariable(SrcLocation loc) : Argument(NK_UnnamedVariable, loc) {}
+
 void UnnamedVariable::print(std::ostream& os) const {
     os << "_";
 }
@@ -19,4 +21,12 @@ UnnamedVariable* UnnamedVariable::cloning() const {
     return new UnnamedVariable(getSrcLoc());
 }
 
+bool UnnamedVariable::classof(const Node* n) {
+    return n->getKind() == NK_UnnamedVariable;
+}
+
+bool UnnamedVariable::equal(const Node&) const {
+    return true;
+}
+
 }  // namespace souffle::ast
