@@ -29,10 +29,16 @@ class UnnamedVariable : public Argument {
 public:
     using Argument::Argument;
 
+    UnnamedVariable(SrcLocation loc = {});
+
+    static bool classof(const Node*);
+
 protected:
     void print(std::ostream& os) const override;
 
 private:
+    bool equal(const Node&) const override;
+
     UnnamedVariable* cloning() const override;
 };
 
