@@ -51,6 +51,20 @@ class TypeEnvironment;
  */
 class Type {
 public:
+    /// LLVM-style no-RTTI dynamic cast
+    // clang-format off
+    enum TypeKind {
+      TK_ConstantType,
+      TK_SubsetType,
+        TK_PrimitiveType,
+      TK_LastSubsetType,
+      TK_AliasType,
+      TK_UnionType,
+      TK_RecordType,
+      TK_AlgebraicDataType
+    };
+    // clang-format on
+
     Type(const Type& other) = delete;
 
     virtual ~Type() = default;
@@ -77,7 +91,7 @@ public:
     }
 
     bool operator<(const Type& other) const {
-        return name < other.name;
+        return name.lexicalLess(other.name);
     }
 
     virtual void print(std::ostream& out) const {
@@ -88,10 +102,18 @@ public:
         return t.print(out), out;
     }
 
+    TypeKind getKind() const {
+        return kind;
+    }
+
+private:
+    const TypeKind kind;
+
 protected:
-    Type(const TypeEnvironment& environment, QualifiedName name,
+    explicit Type(TypeKind tyKind, const TypeEnvironment& environment, QualifiedName name,
             std::optional<QualifiedName> maybePrettyName = std::nullopt)
-            : environment(environment), name(std::move(name)), maybePrettyName(std::move(maybePrettyName)) {}
+            : kind(tyKind), environment(environment), name(std::move(name)),
+              maybePrettyName(std::move(maybePrettyName)) {}
 
     /** Type environment of type */
     const TypeEnvironment& environment;
@@ -113,7 +135,12 @@ protected:
 class ConstantType : public Type {
     ConstantType(const TypeEnvironment& environment, const QualifiedName& name,
             std::optional<QualifiedName> maybePrettyName = std::nullopt)
-            : Type(environment, name, std::move(maybePrettyName)) {}
+            : Type(TK_ConstantType, environment, name, std::move(maybePrettyName)) {}
+
+public:
+    static bool classof(const Type* t) {
+        return t->getKind() == TK_ConstantType;
+    }
 
 private:
     friend class TypeEnvironment;
@@ -129,7 +156,7 @@ private:
  *
  * where T is a type.
  */
-class SubsetType : virtual public Type {
+class SubsetType : public Type {
 public:
     void print(std::ostream& out) const override;
 
@@ -137,9 +164,16 @@ public:
         return baseType;
     }
 
+    static bool classof(const Type* t) {
+        return t->getKind() >= TK_SubsetType && t->getKind() < TK_LastSubsetType;
+    }
+
+    SubsetType(TypeKind kind, const TypeEnvironment& environment, const QualifiedName& name, const Type& base)
+            : Type(kind, environment, name), baseType(base) {}
+
 protected:
     SubsetType(const TypeEnvironment& environment, const QualifiedName& name, const Type& base)
-            : Type(environment, name), baseType(base){};
+            : SubsetType(TK_SubsetType, environment, name, base) {}
 
 private:
     friend class TypeEnvironment;
@@ -166,9 +200,13 @@ public:
         return aliasType;
     }
 
+    static bool classof(const Type* t) {
+        return t->getKind() == TK_AliasType;
+    }
+
 protected:
     AliasType(const TypeEnvironment& environment, const QualifiedName& name, const Type& alias)
-            : Type(environment, name), aliasType(alias){};
+            : Type(TK_AliasType, environment, name), aliasType(alias){};
 
 private:
     friend class TypeEnvironment;
@@ -194,9 +232,13 @@ public:
         out << name;
     }
 
+    static bool classof(const Type* t) {
+        return t->getKind() == TK_PrimitiveType;
+    }
+
 protected:
     PrimitiveType(const TypeEnvironment& environment, const QualifiedName& name, const ConstantType& base)
-            : Type(environment, name), SubsetType(environment, name, base) {}
+            : SubsetType(TK_PrimitiveType, environment, name, base) {}
 
 private:
     friend class TypeEnvironment;
@@ -225,10 +267,14 @@ public:
 
     void print(std::ostream& out) const override;
 
+    static bool classof(const Type* t) {
+        return t->getKind() == TK_UnionType;
+    }
+
 protected:
     UnionType(const TypeEnvironment& environment, const QualifiedName& name,
             std::vector<const Type*> elementTypes = {})
-            : Type(environment, name), elementTypes(std::move(elementTypes)) {}
+            : Type(TK_UnionType, environment, name), elementTypes(std::move(elementTypes)) {}
 
 private:
     friend class TypeEnvironment;
@@ -259,10 +305,14 @@ public:
 
     void print(std::ostream& out) const override;
 
+    static bool classof(const Type* t) {
+        return t->getKind() == TK_RecordType;
+    }
+
 protected:
     RecordType(const TypeEnvironment& environment, const QualifiedName& name,
             const std::vector<const Type*> fields = {})
-            : Type(environment, name), fields(fields) {}
+            : Type(TK_RecordType, environment, name), fields(fields) {}
 
 private:
     friend class TypeEnvironment;
@@ -311,15 +361,15 @@ public:
     void setBranches(std::vector<Branch> bs) {
         branches = std::move(bs);
         std::sort(branches.begin(), branches.end(),
-                [](const Branch& left, const Branch& right) { return left.name < right.name; });
+                [](const Branch& left, const Branch& right) { return left.name.lexicalLess(right.name); });
     }
 
-    const std::vector<const Type*>& getBranchTypes(const QualifiedName& name) const {
+    std::vector<const Type*> getBranchTypes(const QualifiedName& name) const {
         for (auto& branch : branches) {
             if (branch.name == name) return branch.types;
         }
         // Branch doesn't exist.
-        throw std::out_of_range("Trying to access non-existing branch.");
+        return {};
     }
 
     /** Return the branches as a sorted vector */
@@ -327,8 +377,13 @@ public:
         return branches;
     }
 
+    static bool classof(const Type* t) {
+        return t->getKind() == TK_AlgebraicDataType;
+    }
+
 protected:
-    AlgebraicDataType(const TypeEnvironment& env, QualifiedName name) : Type(env, std::move(name)) {}
+    AlgebraicDataType(const TypeEnvironment& env, QualifiedName name)
+            : Type(TK_AlgebraicDataType, env, std::move(name)) {}
 
 private:
     friend class TypeEnvironment;
@@ -468,7 +523,9 @@ public:
     /** Print type set */
     void print(std::ostream& out) const {
         if (all) {
-            out << "{ - all types - }";
+            out << "⊤";
+        } else if (types.empty()) {
+            out << "∅";
         } else {
             out << "{"
                 << join(types, ",", [](std::ostream& out, const Type* type) { out << type->getName(); })
@@ -518,12 +575,12 @@ public:
 
     const Type& getConstantType(TypeAttribute type) const {
         switch (type) {
-            case TypeAttribute::Signed: return getType("__numberConstant");
-            case TypeAttribute::Unsigned: return getType("__unsignedConstant");
-            case TypeAttribute::Float: return getType("__floatConstant");
-            case TypeAttribute::Symbol: return getType("__symbolConstant");
-            case TypeAttribute::Record: return getType("__numberConstant");
-            case TypeAttribute::ADT: return getType("__numberConstant");
+            case TypeAttribute::Signed: return getType(QualifiedName::fromString("__numberConstant"));
+            case TypeAttribute::Unsigned: return getType(QualifiedName::fromString("__unsignedConstant"));
+            case TypeAttribute::Float: return getType(QualifiedName::fromString("__floatConstant"));
+            case TypeAttribute::Symbol: return getType(QualifiedName::fromString("__symbolConstant"));
+            case TypeAttribute::Record: return getType(QualifiedName::fromString("__numberConstant"));
+            case TypeAttribute::ADT: return getType(QualifiedName::fromString("__numberConstant"));
         }
         fatal("Unhandled type");
     }
@@ -571,11 +628,12 @@ private:
     TypeSet initializeConstantTypes();
 
     /** Map for storing named types */
-    std::map<QualifiedName, Own<Type>> types;
+    UnorderedQualifiedNameMap<Own<Type>> types;
 
     const TypeSet constantTypes = initializeConstantTypes();
-    const TypeSet constantNumericTypes =
-            TypeSet(getType("__numberConstant"), getType("__unsignedConstant"), getType("__floatConstant"));
+    const TypeSet constantNumericTypes = TypeSet(getType(QualifiedName::fromString("__numberConstant")),
+            getType(QualifiedName::fromString("__unsignedConstant")),
+            getType(QualifiedName::fromString("__floatConstant")));
 
     const TypeSet primitiveTypes = initializePrimitiveTypes();
 };
