@@ -23,6 +23,7 @@
 #include "ast/ComponentInit.h"
 #include "ast/ComponentType.h"
 #include "ast/Directive.h"
+#include "ast/Lattice.h"
 #include "ast/Node.h"
 #include "ast/Program.h"
 #include "ast/QualifiedName.h"
@@ -58,6 +59,7 @@ static const unsigned int MAX_INSTANTIATION_DEPTH = 100;
  */
 struct ComponentContent {
     VecOwn<ast::Type> types;
+    VecOwn<ast::Lattice> lattices;
     VecOwn<Relation> relations;
     VecOwn<Directive> directives;
     VecOwn<Clause> clauses;
@@ -77,6 +79,22 @@ struct ComponentContent {
         types.push_back(std::move(type));
     }
 
+    void add(Own<ast::Lattice>& lattice, ErrorReport& report) {
+        // add to result content (check existence first)
+        auto foundItem =
+                std::find_if(lattices.begin(), lattices.end(), [&](const Own<ast::Lattice>& element) {
+                    return (element->getQualifiedName() == lattice->getQualifiedName());
+                });
+        if (foundItem != lattices.end()) {
+            Diagnostic err(Diagnostic::Type::ERROR,
+                    DiagnosticMessage("Redefinition of lattice " + toString(lattice->getQualifiedName()),
+                            lattice->getSrcLoc()),
+                    {DiagnosticMessage("Previous definition", (*foundItem)->getSrcLoc())});
+            report.addDiagnostic(err);
+        }
+        lattices.push_back(std::move(lattice));
+    }
+
     void add(Own<Relation>& rel, ErrorReport& report) {
         // add to result content (check existence first)
         auto foundItem = std::find_if(relations.begin(), relations.end(), [&](const Own<Relation>& element) {
@@ -164,6 +182,11 @@ void collectContent(Program& program, const Component& component, const TypeBind
                     res.add(type, report);
                 }
 
+                // process lattices
+                for (auto& lattice : content.lattices) {
+                    res.add(lattice, report);
+                }
+
                 // process relations
                 for (auto& rel : content.relations) {
                     res.add(rel, report);
@@ -256,6 +279,17 @@ void collectContent(Program& program, const Component& component, const TypeBind
         res.add(type, report);
     }
 
+    for (const auto& cur : component.getLattices()) {
+        // create a clone
+        Own<ast::Lattice> lattice(clone(cur));
+
+        auto&& newName = binding.find(lattice->getQualifiedName());
+        if (!newName.empty()) {
+            lattice->setQualifiedName(newName);
+        }
+        res.add(lattice, report);
+    }
+
     // and the local relations
     // (replacing formal parameters with actual parameters)
     for (const auto& cur : component.getRelations()) {
@@ -283,7 +317,7 @@ void collectContent(Program& program, const Component& component, const TypeBind
     }
 
     // index the available relations
-    std::map<QualifiedName, Relation*> index;
+    UnorderedQualifiedNameMap<Relation*> index;
     for (const auto& cur : res.relations) {
         index[cur->getQualifiedName()] = cur.get();
     }
@@ -336,6 +370,8 @@ ComponentContent getInstantiatedContent(Program& program, const ComponentInit& c
 
     if (component == nullptr) {
         // this component is not defined => will trigger a semantic error
+        report.addError("Component " + componentInit.getComponentType()->getName() + " not found",
+                componentInit.getComponentType()->getSrcLoc());
         return res;
     }
 
@@ -355,6 +391,11 @@ ComponentContent getInstantiatedContent(Program& program, const ComponentInit& c
             res.add(type, report);
         }
 
+        // add types
+        for (auto& lattice : nestedContent.lattices) {
+            res.add(lattice, report);
+        }
+
         // add relations
         for (auto& rel : nestedContent.relations) {
             res.add(rel, report);
@@ -376,7 +417,7 @@ ComponentContent getInstantiatedContent(Program& program, const ComponentInit& c
             componentLookup, res, orphans, overridden, report, maxDepth);
 
     // update user-defined type names
-    std::map<QualifiedName, QualifiedName> typeNameMapping;
+    UnorderedQualifiedNameMap<QualifiedName> typeNameMapping;
     for (const auto& cur : res.types) {
         auto newName = componentInit.getInstanceName() + cur->getQualifiedName();
         typeNameMapping[cur->getQualifiedName()] = newName;
@@ -389,8 +430,13 @@ ComponentContent getInstantiatedContent(Program& program, const ComponentInit& c
         });
     }
 
+    for (const auto& cur : res.lattices) {
+        auto newName = componentInit.getInstanceName() + cur->getQualifiedName();
+        cur->setQualifiedName(newName);
+    }
+
     // update relation names
-    std::map<QualifiedName, QualifiedName> relationNameMapping;
+    UnorderedQualifiedNameMap<QualifiedName> relationNameMapping;
     for (const auto& cur : res.relations) {
         auto newName = componentInit.getInstanceName() + cur->getQualifiedName();
         relationNameMapping[cur->getQualifiedName()] = newName;
@@ -541,7 +587,7 @@ bool ComponentInstantiationTransformer::transform(TranslationUnit& translationUn
 
     auto& componentLookup = translationUnit.getAnalysis<ComponentLookupAnalysis>();
 
-    for (const auto* cur : program.getComponentInstantiations()) {
+    for (const auto* cur : program.getInstantiations()) {
         VecOwn<Clause> orphans;
         const std::set<std::string> overridden;
 
@@ -552,6 +598,9 @@ bool ComponentInstantiationTransformer::transform(TranslationUnit& translationUn
         for (auto& type : content.types) {
             program.addType(std::move(type));
         }
+        for (auto& lattice : content.lattices) {
+            program.addLattice(std::move(lattice));
+        }
         for (auto& rel : content.relations) {
             program.addRelation(std::move(rel));
         }
