@@ -14,6 +14,7 @@
 
 #include "ast/transform/MaterializeAggregationQueries.h"
 #include "AggregateOp.h"
+#include "FunctorOps.h"
 #include "ast/Aggregator.h"
 #include "ast/Argument.h"
 #include "ast/Atom.h"
@@ -138,7 +139,7 @@ void MaterializeAggregationQueriesTransformer::groundInjectedParameters(
     };
 
     auto aggClauseInnerAggregatesMasked = clone(aggClause);
-    aggClauseInnerAggregatesMasked->setHead(mk<Atom>("*"));
+    aggClauseInnerAggregatesMasked->setHead(mk<Atom>(QualifiedName::fromString("*")));
     NegateAggregateAtoms update;
     aggClauseInnerAggregatesMasked->apply(update);
 
@@ -192,7 +193,7 @@ void MaterializeAggregationQueriesTransformer::groundInjectedParameters(
                 continue;
             }
             // 2. Variable must be grounded by this literal.
-            auto singleLiteralClause = mk<Clause>("*");
+            auto singleLiteralClause = mk<Clause>(QualifiedName::fromString("*"));
             singleLiteralClause->addToBody(clone(lit));
             bool variableGroundedByLiteral = false;
             for (const auto& argPair : analysis::getGroundedTerms(translationUnit, *singleLiteralClause)) {
@@ -272,8 +273,15 @@ bool MaterializeAggregationQueriesTransformer::materializeAggregationQueries(
         return true;
     });
 
+    // keep literals alive while we use the type analysis, issue (#1896).
+    VecOwn<Literal> oldBodyLiterals;
+
     for (auto&& cl : program.getClauses()) {
         auto& clause = *cl;
+
+        // the mapping from the Arguments in the original clause to their type(s)
+        std::optional<std::map<const Argument*, analysis::TypeSet>> clauseArgTypes;
+
         visit(clause, [&](Aggregator& agg) {
             if (!needsMaterializedRelation(agg)) {
                 return;
@@ -282,10 +290,16 @@ bool MaterializeAggregationQueriesTransformer::materializeAggregationQueries(
             if (innerAggregates.find(&agg) != innerAggregates.end()) {
                 return;
             }
+
+            // compute types before the clause gets modified
+            if (!clauseArgTypes) {
+                clauseArgTypes = analysis::TypeAnalysis::analyseTypes(translationUnit, clause);
+            }
+
             // begin materialisation process
             auto aggregateBodyRelationName = analysis::findUniqueRelationName(program, "__agg_subclause");
             // quickly copy in all the literals from the aggregate body
-            auto aggClause = mk<Clause>(aggregateBodyRelationName);
+            auto aggClause = mk<Clause>(QualifiedName::fromString(aggregateBodyRelationName));
             aggClause->setBodyLiterals(clone(agg.getBodyLiterals()));
             if (const auto* intrinsicAgg = as<IntrinsicAggregator>(agg)) {
                 if (intrinsicAgg->getBaseOperator() == AggregateOp::COUNT) {
@@ -303,17 +317,28 @@ bool MaterializeAggregationQueriesTransformer::materializeAggregationQueries(
             for (const auto& variableName : headArguments) {
                 aggClauseHead->addArgument(mk<Variable>(variableName));
             }
-            // add them to the relation as well (need to do a bit of type analysis to make this work)
-            auto aggRel = mk<Relation>(aggregateBodyRelationName);
-            std::map<const Argument*, analysis::TypeSet> argTypes =
-                    analysis::TypeAnalysis::analyseTypes(translationUnit, *aggClause);
+
+            // add them to the relation as well
+            auto aggRel = mk<Relation>(QualifiedName::fromString(aggregateBodyRelationName));
 
             for (const auto& cur : aggClauseHead->getArguments()) {
-                // cur will point us to a particular argument
-                // that is found in the aggClause
-                auto const curArgType = argTypes[cur];
-                assert(!curArgType.empty() && "unexpected empty typeset");
-                aggRel->addAttribute(mk<Attribute>(toString(*cur), curArgType.begin()->getName()));
+                // Find type of argument variable in original clause
+                auto it = std::find_if(clauseArgTypes->cbegin(), clauseArgTypes->cend(),
+                        [&](const std::pair<const Argument*, analysis::TypeSet>& pair) -> bool {
+                            if (const Variable* var = as<Variable>(pair.first)) {
+                                // use type from first variable matching the name
+                                return (var->getName() == toString(*cur));
+                            }
+                            return false;
+                        });
+                assert(it != clauseArgTypes->cend() && "unexpected unknown argument");
+
+                if (it != clauseArgTypes->cend()) {
+                    auto const curArgType = it->second;
+                    assert(!curArgType.empty() && "unexpected empty typeset");
+                    assert(curArgType.size() == 1 && "expected fully resolved type");
+                    aggRel->addAttribute(mk<Attribute>(toString(*cur), curArgType.begin()->getName()));
+                }
             }
 
             // Set up the aggregate body atom that will represent the materialised relation we just created
@@ -345,7 +370,9 @@ bool MaterializeAggregationQueriesTransformer::materializeAggregationQueries(
 
             VecOwn<Literal> newBody;
             newBody.push_back(std::move(aggAtom));
-            agg.setBodyLiterals(std::move(newBody));
+            VecOwn<Literal> oldBody = agg.setBodyLiterals(std::move(newBody));
+            oldBodyLiterals.insert(oldBodyLiterals.end(), std::make_move_iterator(oldBody.begin()),
+                    std::make_move_iterator(oldBody.end()));
             // Now we can just add these new things (relation and its single clause) to the program
             program.addClause(std::move(aggClause));
             program.addRelation(std::move(aggRel));
@@ -383,6 +410,19 @@ bool MaterializeAggregationQueriesTransformer::needsMaterializedRelation(const A
         return true;
     }
 
+    // If we have a multi-result intrinsic functor within this aggregate => materialize
+    bool seenMultiresultIntrinsicFunctor = false;
+    visit(agg, [&](const IntrinsicFunctor& intFunc) {
+        auto candidates = functorBuiltIn(intFunc.getBaseFunctionOp());
+        seenMultiresultIntrinsicFunctor |= std::any_of(candidates.cbegin(), candidates.cend(),
+                [](const std::reference_wrapper<const IntrinsicFunctorInfo>& info) -> bool {
+                    return isFunctorMultiResult(info.get().op);
+                });
+    });
+    if (seenMultiresultIntrinsicFunctor) {
+        return true;
+    }
+
     // If the same variable occurs several times => materialize
     bool duplicates = false;
     std::set<std::string> vars;
