@@ -16,20 +16,20 @@
 
 #include "ast/transform/Meta.h"
 #include "souffle/utility/MiscUtil.h"
-#include <chrono>
 #include <iostream>
 
 namespace souffle::ast::transform {
 
 bool MetaTransformer::applySubtransformer(TranslationUnit& translationUnit, Transformer* transformer) {
-    auto start = std::chrono::high_resolution_clock::now();
+    auto start = now();
     bool changed = transformer->apply(translationUnit);
-    auto end = std::chrono::high_resolution_clock::now();
+    auto end = now();
 
     if (verbose && (!isA<MetaTransformer>(transformer))) {
         std::string changedString = changed ? "changed" : "unchanged";
-        std::cout << transformer->getName() << " time: " << std::chrono::duration<double>(end - start).count()
-                  << "sec [" << changedString << "]" << std::endl;
+        const auto elapsed = duration_in_us(start, end);
+        std::cout << transformer->getName() << " time: " << std::to_string(elapsed / 1000000.0) << "s ["
+                  << changedString << "]" << std::endl;
     }
 
     return changed;
