@@ -38,6 +38,7 @@
 #include "ast/IntrinsicAggregator.h"
 #include "ast/IntrinsicFunctor.h"
 #include "ast/IterationCounter.h"
+#include "ast/Lattice.h"
 #include "ast/Literal.h"
 #include "ast/Negation.h"
 #include "ast/NilConstant.h"
@@ -124,6 +125,7 @@ private:
     void checkRelationFunctionalDependencies(const Relation& relation);
     void checkRelation(const Relation& relation);
     void checkType(ast::Attribute const& attr, std::string const& name = {});
+    void checkLatticeDeclaration(const Lattice& lattice);
     void checkFunctorDeclaration(const FunctorDeclaration& decl);
 
     void checkNamespaces();
@@ -152,13 +154,9 @@ SemanticCheckerImpl::SemanticCheckerImpl(TranslationUnit& tu) : tu(tu) {
         } else {
             // mute only the given relations (if they exist)
             for (auto& relname : suppressedRelations) {
-                const std::vector<std::string> comps = splitString(relname, '.');
-                if (!comps.empty()) {
+                if (!relname.empty()) {
                     // generate the relation identifier
-                    QualifiedName relid(comps[0]);
-                    for (std::size_t i = 1; i < comps.size(); i++) {
-                        relid.append(comps[i]);
-                    }
+                    QualifiedName relid = QualifiedName::fromString(relname);
 
                     // update suppressed qualifier if the relation is found
                     if (Relation* rel = program.getRelation(relid)) {
@@ -173,6 +171,9 @@ SemanticCheckerImpl::SemanticCheckerImpl(TranslationUnit& tu) : tu(tu) {
     for (auto* rel : program.getRelations()) {
         checkRelation(*rel);
     }
+    for (auto* lattice : program.getLattices()) {
+        checkLatticeDeclaration(*lattice);
+    }
     for (auto* clause : program.getClauses()) {
         checkClause(*clause);
     }
@@ -207,19 +208,19 @@ SemanticCheckerImpl::SemanticCheckerImpl(TranslationUnit& tu) : tu(tu) {
 
     // - stratification --
     // check for cyclic dependencies
-    for (Relation* cur : program.getRelations()) {
+    for (const Relation* cur : program.getRelations()) {
         std::size_t scc = sccGraph.getSCC(cur);
         if (sccGraph.isRecursive(scc)) {
-            for (const Relation* cyclicRelation : sccGraph.getInternalRelations(scc)) {
+            const RelationSet& relSet = sccGraph.getInternalRelations(scc);
+            for (const Relation* cyclicRelation : relSet) {
                 // Negations and aggregations need to be stratified
                 const Literal* foundLiteral = nullptr;
                 bool hasNegation = hasClauseWithNegatedRelation(cyclicRelation, cur, &program, foundLiteral);
-                if (hasNegation ||
-                        hasClauseWithAggregatedRelation(cyclicRelation, cur, &program, foundLiteral)) {
-                    auto const& relSet = sccGraph.getInternalRelations(scc);
-                    RelationSet sortedRelSet(relSet.begin(), relSet.end());
+                bool hasAggregate =
+                        hasClauseWithAggregatedRelation(cyclicRelation, cur, &program, foundLiteral);
+                if (hasNegation || hasAggregate) {
                     // Negations and aggregations need to be stratified
-                    std::string relationsListStr = toString(join(sortedRelSet, ",",
+                    std::string relationsListStr = toString(join(relSet, ",",
                             [](std::ostream& out, const Relation* r) { out << r->getQualifiedName(); }));
                     std::vector<DiagnosticMessage> messages;
                     messages.push_back(DiagnosticMessage(
@@ -244,10 +245,10 @@ void SemanticCheckerImpl::checkAtom(const Clause& parent, const Atom& atom) {
         report.addError("Undefined relation " + toString(atom.getQualifiedName()), atom.getSrcLoc());
         return;
     }
-
-    if (r->getArity() != atom.getArity()) {
+    std::size_t arity = r->getArity();
+    if (arity != atom.getArity()) {
         report.addError("Mismatching arity of relation " + toString(atom.getQualifiedName()) + " (expected " +
-                                toString(r->getArity()) + ", got " + toString(atom.getArity()) + ")",
+                                toString(arity) + ", got " + toString(atom.getArity()) + ")",
                 atom.getSrcLoc());
     }
 
@@ -350,7 +351,8 @@ bool SemanticCheckerImpl::isDependent(const Clause& agg1, const Clause& agg2) {
 
 void SemanticCheckerImpl::checkAggregator(const Clause& parent, const Aggregator& aggregator) {
     auto& report = tu.getErrorReport();
-    Clause dummyClauseAggregator("dummy");
+    const QualifiedName dummyQN = QualifiedName::fromString("dummy");
+    Clause dummyClauseAggregator(dummyQN);
 
     visit(parent, [&](const Literal& parentLiteral) {
         visit(parentLiteral, [&](const Aggregator& candidateAggregate) {
@@ -366,7 +368,7 @@ void SemanticCheckerImpl::checkAggregator(const Clause& parent, const Aggregator
     visit(parent, [&](const Literal& parentLiteral) {
         visit(parentLiteral, [&](const Aggregator& /* otherAggregate */) {
             // Create the other aggregate's dummy clause
-            Clause dummyClauseOther("dummy");
+            Clause dummyClauseOther(dummyQN);
             dummyClauseOther.addToBody(clone(parentLiteral));
             // Check dependency between the aggregator and this one
             if (isDependent(dummyClauseAggregator, dummyClauseOther) &&
@@ -658,11 +660,43 @@ void SemanticCheckerImpl::checkFunctorDeclaration(const FunctorDeclaration& decl
     }
 }
 
+void SemanticCheckerImpl::checkLatticeDeclaration(const Lattice& lattice) {
+    const auto& name = lattice.getQualifiedName();
+    auto* existingType = getIf(
+            program.getTypes(), [&](const ast::Type* type) { return type->getQualifiedName() == name; });
+    if (!existingType) {
+        report.addError(tfm::format("Undefined type %s", name), lattice.getSrcLoc());
+    }
+    if (lattice.hasLub()) {
+        if (!isA<UserDefinedFunctor>(lattice.getLub())) {
+            report.addError(
+                    tfm::format("Lattice operator Lub must be a user-defined functor"), lattice.getSrcLoc());
+        }
+    } else {
+        report.addError(tfm::format("Lattice %s<> does not define Lub", name), lattice.getSrcLoc());
+    }
+    if (lattice.hasGlb()) {
+        if (!isA<UserDefinedFunctor>(lattice.getGlb())) {
+            report.addError(
+                    tfm::format("Lattice operator Glb must be a user-defined functor"), lattice.getSrcLoc());
+        }
+    } else {
+        report.addWarning(WarnType::LatticeMissingOperator,
+                tfm::format("Lattice %s<> does not define Glb", name), lattice.getSrcLoc());
+    }
+    if (!lattice.hasBottom()) {
+        report.addWarning(WarnType::LatticeMissingOperator,
+                tfm::format("Lattice %s<> does not define Bottom", name), lattice.getSrcLoc());
+    }
+}
+
 void SemanticCheckerImpl::checkRelationDeclaration(const Relation& relation) {
     const auto& attributes = relation.getAttributes();
-    assert(attributes.size() == relation.getArity() && "mismatching attribute size and arity");
+    const std::size_t arity = relation.getArity();
+    std::size_t firstAuxiliary = arity - relation.getAuxiliaryArity();
 
-    for (std::size_t i = 0; i < relation.getArity(); i++) {
+    assert(attributes.size() == arity && "mismatching attribute size and arity");
+    for (std::size_t i = 0; i < arity; i++) {
         Attribute* attr = attributes[i];
         checkType(*attr);
 
@@ -672,6 +706,25 @@ void SemanticCheckerImpl::checkRelationDeclaration(const Relation& relation) {
                 report.addError(tfm::format("Doubly defined attribute name %s", *attr), attr->getSrcLoc());
             }
         }
+
+        /* check that lattice elements are always the last */
+        if (i < firstAuxiliary && attr->getIsLattice()) {
+            report.addError(
+                    tfm::format(
+                            "Lattice attribute %s should be placed after all non-lattice attributes", *attr),
+                    attr->getSrcLoc());
+        }
+
+        /* check that lattice attributes have a correct lattice definition */
+        if (attr->getIsLattice()) {
+            const auto& typeName = attr->getTypeName();
+            auto* existingType = getIf(program.getLattices(),
+                    [&](const ast::Lattice* lattice) { return lattice->getQualifiedName() == typeName; });
+            if (!existingType) {
+                report.addError(
+                        tfm::format("Missing lattice definition for type %s", typeName), attr->getSrcLoc());
+            }
+        }
     }
 }
 
@@ -728,7 +781,21 @@ void SemanticCheckerImpl::checkRelation(const Relation& relation) {
     }
     if (relation.getRepresentation() == RelationRepresentation::BTREE_DELETE && relation.getArity() == 0) {
         report.addError("Subsumptive relation \"" + toString(relation.getQualifiedName()) +
-                                "\"  must not be a nullary relation",
+                                "\" must not be a nullary relation",
+                relation.getSrcLoc());
+    }
+
+    if (hasSubsumptiveRule && relation.getAuxiliaryArity()) {
+        report.addError("Relation \"" + toString(relation.getQualifiedName()) +
+                                "\" must not have both subsumptive rules and lattice arguments",
+                relation.getSrcLoc());
+    }
+
+    if (relation.getAuxiliaryArity() &&
+            (relation.getRepresentation() != RelationRepresentation::BTREE &&
+                    relation.getRepresentation() != RelationRepresentation::DEFAULT)) {
+        report.addError(
+                "Relation \"" + toString(relation.getQualifiedName()) + "\" must have a btree representation",
                 relation.getSrcLoc());
     }
 
@@ -795,7 +862,7 @@ static const std::vector<SrcLocation> usesInvalidWitness(
         return invalidWitnessLocations;  // ie empty result
     }
 
-    auto aggregateSubclause = mk<Clause>("*");
+    auto aggregateSubclause = mk<Clause>(QualifiedName::fromString("*"));
     aggregateSubclause->setBodyLiterals(clone(aggregate.getBodyLiterals()));
 
     struct InnerAggregateMasker : public NodeMapper {
@@ -886,7 +953,7 @@ std::vector<QualifiedName> findInlineCycle(const PrecedenceGraphAnalysis& preced
     }
 
     // Check neighbours
-    const RelationSet& successors = precedenceGraph.graph().successors(current);
+    const RelationSet& successors = orderedRelationSet(precedenceGraph.graph().successors(current));
     for (const Relation* successor : successors) {
         // Only care about inlined neighbours in the graph
         if (successor->hasQualifier(RelationQualifier::INLINE)) {
