@@ -24,12 +24,14 @@
 #include "ast2ram/utility/TranslatorContext.h"
 #include "ast2ram/utility/Utils.h"
 #include "ast2ram/utility/ValueIndex.h"
+#include "ram/AbstractOperator.h"
 #include "ram/Call.h"
 #include "ram/DebugInfo.h"
 #include "ram/ExistenceCheck.h"
 #include "ram/Expression.h"
 #include "ram/Filter.h"
 #include "ram/Insert.h"
+#include "ram/IntrinsicOperator.h"
 #include "ram/LogRelationTimer.h"
 #include "ram/MergeExtend.h"
 #include "ram/Negation.h"
@@ -69,18 +71,12 @@ Own<ram::Sequence> UnitTranslator::generateProgram(const ast::TranslationUnit& t
 
 Own<ram::Relation> UnitTranslator::createRamRelation(
         const ast::Relation* baseRelation, std::string ramRelationName) const {
-    auto arity = baseRelation->getArity();
+    auto relation = seminaive::UnitTranslator::createRamRelation(baseRelation, ramRelationName);
 
-    // All relations in a provenance program should have a provenance data structure
-    auto representation = RelationRepresentation::PROVENANCE;
-
-    // Add in base relation information
-    std::vector<std::string> attributeNames;
-    std::vector<std::string> attributeTypeQualifiers;
-    for (const auto& attribute : baseRelation->getAttributes()) {
-        attributeNames.push_back(attribute->getName());
-        attributeTypeQualifiers.push_back(context->getAttributeTypeQualifier(attribute->getTypeName()));
-    }
+    std::size_t arity = relation->getArity();
+    std::size_t auxiliaryArity = relation->getAuxiliaryArity();
+    std::vector<std::string> attributeNames = relation->getAttributeNames();
+    std::vector<std::string> attributeTypeQualifiers = relation->getAttributeTypes();
 
     // Add in provenance information
     attributeNames.push_back("@rule_number");
@@ -89,8 +85,8 @@ Own<ram::Relation> UnitTranslator::createRamRelation(
     attributeNames.push_back("@level_number");
     attributeTypeQualifiers.push_back("i:number");
 
-    return mk<ram::Relation>(
-            ramRelationName, arity + 2, 2, attributeNames, attributeTypeQualifiers, representation);
+    return mk<ram::Relation>(ramRelationName, arity + 2, auxiliaryArity + 2, attributeNames,
+            attributeTypeQualifiers, relation->getRepresentation());
 }
 
 std::string UnitTranslator::getInfoRelationName(const ast::Clause* clause) const {
@@ -300,7 +296,8 @@ Own<ram::Sequence> UnitTranslator::generateInfoClauses(const ast::Program* progr
         // Add debug info
         std::ostringstream ds;
         ds << "@info.clause[";
-        ds << toString(*clause) << "]"
+        clause->printForDebugInfo(ds);
+        ds << "]"
            << "\nin file ";
         ds << clause->getSrcLoc();
         infoClause = mk<ram::DebugInfo>(std::move(infoClause), ds.str());
