@@ -98,9 +98,9 @@
 #include "souffle/profile/Logger.h"
 #include "souffle/profile/ProfileEvent.h"
 #include "souffle/utility/EvaluatorUtil.h"
-#include "souffle/utility/MiscUtil.h"
 #include "souffle/utility/ParallelUtil.h"
 #include "souffle/utility/StringUtil.h"
+
 #include <algorithm>
 #include <array>
 #include <atomic>
@@ -359,13 +359,13 @@ void Engine::createRelation(const ram::Relation& id, const std::size_t idx) {
     }
 
     RelationHandle res;
-
-    if (id.getRepresentation() == RelationRepresentation::EQREL) {
+    bool hasProvenance = id.getArity() > 0 && id.getAttributeNames().back() == "@level_number";
+    if (hasProvenance) {
+        res = createProvenanceRelation(id, isa.getIndexSelection(id.getName()));
+    } else if (id.getRepresentation() == RelationRepresentation::EQREL) {
         res = createEqrelRelation(id, isa.getIndexSelection(id.getName()));
     } else if (id.getRepresentation() == RelationRepresentation::BTREE_DELETE) {
         res = createBTreeDeleteRelation(id, isa.getIndexSelection(id.getName()));
-    } else if (id.getRepresentation() == RelationRepresentation::PROVENANCE) {
-        res = createProvenanceRelation(id, isa.getIndexSelection(id.getName()));
     } else {
         res = createBTreeRelation(id, isa.getIndexSelection(id.getName()));
     }
@@ -424,9 +424,11 @@ const std::vector<void*>& Engine::loadDLL() {
 std::size_t Engine::getIterationNumber() const {
     return iteration;
 }
+
 void Engine::incIterationNumber() {
     ++iteration;
 }
+
 void Engine::resetIterationNumber() {
     iteration = 0;
 }
@@ -480,6 +482,8 @@ void Engine::executeMain() {
         visit(program, [&](const ram::Query&) { ++ruleCount; });
         ProfileEventSingleton::instance().makeConfigRecord("ruleCount", std::to_string(ruleCount));
 
+        SignalHandler::instance()->enableProfiling();
+
         Context ctxt;
         execute(main.get(), ctxt);
         ProfileEventSingleton::instance().stopTimer();
@@ -527,9 +531,9 @@ RamDomain Engine::execute(const Node* node, Context& ctxt) {
 
 // Overload CASE based on number of arguments.
 // CASE(Kind) -> BASE_CASE(Kind)
-// CASE(Kind, Structure, Arity) -> EXTEND_CASE(Kind, Structure, Arity)
-#define GET_MACRO(_1, _2, _3, NAME, ...) NAME
-#define CASE(...) GET_MACRO(__VA_ARGS__, EXTEND_CASE, _Dummy, BASE_CASE)(__VA_ARGS__)
+// CASE(Kind, Structure, Arity, AuxiliaryArity) -> EXTEND_CASE(Kind, Structure, Arity, AuxiliaryArity)
+#define GET_MACRO(_1, _2, _3, _4, NAME, ...) NAME
+#define CASE(...) GET_MACRO(__VA_ARGS__, EXTEND_CASE, _Dummy, _Dummy2, BASE_CASE)(__VA_ARGS__)
 
 #define BASE_CASE(Kind) \
     case (I_##Kind): {  \
@@ -537,12 +541,12 @@ RamDomain Engine::execute(const Node* node, Context& ctxt) {
             [[maybe_unused]] const auto& shadow = *static_cast<const interpreter::Kind*>(node); \
             [[maybe_unused]] const auto& cur = *static_cast<const ram::Kind*>(node->getShadow());
 // EXTEND_CASE also defer the relation type
-#define EXTEND_CASE(Kind, Structure, Arity)    \
-    case (I_##Kind##_##Structure##_##Arity): { \
+#define EXTEND_CASE(Kind, Structure, Arity, AuxiliaryArity)       \
+    case (I_##Kind##_##Structure##_##Arity##_##AuxiliaryArity): { \
         return [&]() -> RamDomain { \
             [[maybe_unused]] const auto& shadow = *static_cast<const interpreter::Kind*>(node); \
             [[maybe_unused]] const auto& cur = *static_cast<const ram::Kind*>(node->getShadow());\
-            using RelType = Relation<Arity, interpreter::Structure>;
+            using RelType = Relation<Arity, AuxiliaryArity, interpreter::Structure>;
 #define ESAC(Kind) \
     }              \
     ();            \
@@ -593,7 +597,7 @@ RamDomain Engine::execute(const Node* node, Context& ctxt) {
         ESAC(AutoIncrement)
 
         CASE(IntrinsicOperator)
-// clang-format off
+        // clang-format off
 #define BINARY_OP_TYPED(ty, op) return ramBitCast(static_cast<ty>(EVAL_CHILD(ty, 0) op EVAL_CHILD(ty, 1)))
 
 #define BINARY_OP_LOGICAL(opcode, op) BINARY_OP_INTEGRAL(opcode, op)
@@ -614,7 +618,7 @@ RamDomain Engine::execute(const Node* node, Context& ctxt) {
     {                                                            \
         auto result = EVAL_CHILD(RamDomain, 0);                  \
         auto* result_val = &getSymbolTable().decode(result);     \
-        for (std::size_t i = 1; i < args.size(); i++) {          \
+        for (std::size_t i = 1; i < numArgs; i++) {          \
             auto alt = EVAL_CHILD(RamDomain, i);                 \
             if (alt == result) continue;                         \
                                                                  \
@@ -629,7 +633,7 @@ RamDomain Engine::execute(const Node* node, Context& ctxt) {
 #define MINMAX_OP(ty, op)                           \
     {                                               \
         auto result = EVAL_CHILD(ty, 0);            \
-        for (std::size_t i = 1; i < args.size(); i++) {  \
+        for (std::size_t i = 1; i < numArgs; i++) {  \
             result = op(result, EVAL_CHILD(ty, i)); \
         }                                           \
         return ramBitCast(result);                  \
@@ -651,7 +655,7 @@ RamDomain Engine::execute(const Node* node, Context& ctxt) {
         getSymbolTable().decode(EVAL_CHILD(RamDomain, 0))));
             // clang-format on
 
-            const auto& args = cur.getArguments();
+            const auto numArgs = cur.getNumArgs();
             switch (cur.getOperator()) {
                 /** Unary Functor Operators */
                 case FunctorOp::ORD: return execute(shadow.getChild(0), ctxt);
@@ -759,7 +763,7 @@ RamDomain Engine::execute(const Node* node, Context& ctxt) {
 
                 case FunctorOp::CAT: {
                     std::stringstream ss;
-                    for (std::size_t i = 0; i < args.size(); i++) {
+                    for (std::size_t i = 0; i < numArgs; i++) {
                         ss << getSymbolTable().decode(execute(shadow.getChild(i), ctxt));
                     }
                     return getSymbolTable().encode(ss.str());
@@ -773,7 +777,7 @@ RamDomain Engine::execute(const Node* node, Context& ctxt) {
                     std::string sub_str;
                     try {
                         sub_str = str.substr(idx, len);
-                    } catch (...) {
+                    } catch (std::out_of_range&) {
                         std::cerr << "warning: wrong index position provided by substr(\"";
                         std::cerr << str << "\"," << (int32_t)idx << "," << (int32_t)len << ") functor.\n";
                     }
@@ -810,8 +814,8 @@ RamDomain Engine::execute(const Node* node, Context& ctxt) {
         ESAC(IntrinsicOperator)
 
         CASE(NestedIntrinsicOperator)
-            auto numArgs = cur.getArguments().size();
-            auto runNested = [&](auto&& tuple) {
+            const auto numArgs = cur.getNumArgs();
+            const auto runNested = [&](auto&& tuple) {
                 ctxt[cur.getTupleId()] = tuple.data();
                 execute(shadow.getChild(numArgs), ctxt);
             };
@@ -837,7 +841,7 @@ RamDomain Engine::execute(const Node* node, Context& ctxt) {
 
             auto userFunctor = reinterpret_cast<void (*)()>(shadow.getFunctionPointer());
             if (userFunctor == nullptr) fatal("cannot find user-defined operator `%s`", name);
-            std::size_t arity = cur.getArguments().size();
+            std::size_t arity = cur.getNumArgs();
 
             if (cur.isStateful()) {
                 auto exec = std::bind(&Engine::execute, this, std::placeholders::_1, std::placeholders::_2);
@@ -957,8 +961,7 @@ RamDomain Engine::execute(const Node* node, Context& ctxt) {
         ESAC(UserDefinedOperator)
 
         CASE(PackRecord)
-            auto values = cur.getArguments();
-            std::size_t arity = values.size();
+            const std::size_t arity = cur.getNumArgs();
             std::unique_ptr<RamDomain[]> data = std::make_unique<RamDomain[]>(arity);
             for (std::size_t i = 0; i < arity; ++i) {
                 data[i] = execute(shadow.getChild(i), ctxt);
@@ -979,15 +982,20 @@ RamDomain Engine::execute(const Node* node, Context& ctxt) {
         ESAC(False)
 
         CASE(Conjunction)
-            return execute(shadow.getLhs(), ctxt) && execute(shadow.getRhs(), ctxt);
+            for (const auto& child : shadow.getChildren()) {
+                if (!execute(child.get(), ctxt)) {
+                    return false;
+                }
+            }
+            return true;
         ESAC(Conjunction)
 
         CASE(Negation)
             return !execute(shadow.getChild(), ctxt);
         ESAC(Negation)
 
-#define EMPTINESS_CHECK(Structure, Arity, ...)                          \
-    CASE(EmptinessCheck, Structure, Arity)                              \
+#define EMPTINESS_CHECK(Structure, Arity, AuxiliaryArity, ...)          \
+    CASE(EmptinessCheck, Structure, Arity, AuxiliaryArity)              \
         const auto& rel = *static_cast<RelType*>(shadow.getRelation()); \
         return rel.empty();                                             \
     ESAC(EmptinessCheck)
@@ -995,8 +1003,8 @@ RamDomain Engine::execute(const Node* node, Context& ctxt) {
         FOR_EACH(EMPTINESS_CHECK)
 #undef EMPTINESS_CHECK
 
-#define RELATION_SIZE(Structure, Arity, ...)                            \
-    CASE(RelationSize, Structure, Arity)                                \
+#define RELATION_SIZE(Structure, Arity, AuxiliaryArity, ...)            \
+    CASE(RelationSize, Structure, Arity, AuxiliaryArity)                \
         const auto& rel = *static_cast<RelType*>(shadow.getRelation()); \
         return rel.size();                                              \
     ESAC(RelationSize)
@@ -1004,17 +1012,17 @@ RamDomain Engine::execute(const Node* node, Context& ctxt) {
         FOR_EACH(RELATION_SIZE)
 #undef RELATION_SIZE
 
-#define EXISTENCE_CHECK(Structure, Arity, ...)            \
-    CASE(ExistenceCheck, Structure, Arity)                \
-        return evalExistenceCheck<RelType>(shadow, ctxt); \
+#define EXISTENCE_CHECK(Structure, Arity, AuxiliaryArity, ...) \
+    CASE(ExistenceCheck, Structure, Arity, AuxiliaryArity)     \
+        return evalExistenceCheck<RelType>(shadow, ctxt);      \
     ESAC(ExistenceCheck)
 
         FOR_EACH(EXISTENCE_CHECK)
 #undef EXISTENCE_CHECK
 
-#define PROVENANCE_EXISTENCE_CHECK(Structure, Arity, ...)           \
-    CASE(ProvenanceExistenceCheck, Structure, Arity)                \
-        return evalProvenanceExistenceCheck<RelType>(shadow, ctxt); \
+#define PROVENANCE_EXISTENCE_CHECK(Structure, Arity, AuxiliaryArity, ...) \
+    CASE(ProvenanceExistenceCheck, Structure, Arity, AuxiliaryArity)      \
+        return evalProvenanceExistenceCheck<RelType>(shadow, ctxt);       \
     ESAC(ProvenanceExistenceCheck)
 
         FOR_EACH_PROVENANCE(PROVENANCE_EXISTENCE_CHECK)
@@ -1135,8 +1143,8 @@ RamDomain Engine::execute(const Node* node, Context& ctxt) {
             return result;
         ESAC(TupleOperation)
 
-#define SCAN(Structure, Arity, ...)                                     \
-    CASE(Scan, Structure, Arity)                                        \
+#define SCAN(Structure, Arity, AuxiliaryArity, ...)                     \
+    CASE(Scan, Structure, Arity, AuxiliaryArity)                        \
         const auto& rel = *static_cast<RelType*>(shadow.getRelation()); \
         return evalScan(rel, cur, shadow, ctxt);                        \
     ESAC(Scan)
@@ -1144,24 +1152,24 @@ RamDomain Engine::execute(const Node* node, Context& ctxt) {
         FOR_EACH(SCAN)
 #undef SCAN
 
-#define PARALLEL_SCAN(Structure, Arity, ...)                            \
-    CASE(ParallelScan, Structure, Arity)                                \
+#define PARALLEL_SCAN(Structure, Arity, AuxiliaryArity, ...)            \
+    CASE(ParallelScan, Structure, Arity, AuxiliaryArity)                \
         const auto& rel = *static_cast<RelType*>(shadow.getRelation()); \
         return evalParallelScan(rel, cur, shadow, ctxt);                \
     ESAC(ParallelScan)
         FOR_EACH(PARALLEL_SCAN)
 #undef PARALLEL_SCAN
 
-#define INDEX_SCAN(Structure, Arity, ...)                 \
-    CASE(IndexScan, Structure, Arity)                     \
+#define INDEX_SCAN(Structure, Arity, AuxiliaryArity, ...) \
+    CASE(IndexScan, Structure, Arity, AuxiliaryArity)     \
         return evalIndexScan<RelType>(cur, shadow, ctxt); \
     ESAC(IndexScan)
 
         FOR_EACH(INDEX_SCAN)
 #undef INDEX_SCAN
 
-#define PARALLEL_INDEX_SCAN(Structure, Arity, ...)                      \
-    CASE(ParallelIndexScan, Structure, Arity)                           \
+#define PARALLEL_INDEX_SCAN(Structure, Arity, AuxiliaryArity, ...)      \
+    CASE(ParallelIndexScan, Structure, Arity, AuxiliaryArity)           \
         const auto& rel = *static_cast<RelType*>(shadow.getRelation()); \
         return evalParallelIndexScan(rel, cur, shadow, ctxt);           \
     ESAC(ParallelIndexScan)
@@ -1169,8 +1177,8 @@ RamDomain Engine::execute(const Node* node, Context& ctxt) {
         FOR_EACH(PARALLEL_INDEX_SCAN)
 #undef PARALLEL_INDEX_SCAN
 
-#define IFEXISTS(Structure, Arity, ...)                                 \
-    CASE(IfExists, Structure, Arity)                                    \
+#define IFEXISTS(Structure, Arity, AuxiliaryArity, ...)                 \
+    CASE(IfExists, Structure, Arity, AuxiliaryArity)                    \
         const auto& rel = *static_cast<RelType*>(shadow.getRelation()); \
         return evalIfExists(rel, cur, shadow, ctxt);                    \
     ESAC(IfExists)
@@ -1178,8 +1186,8 @@ RamDomain Engine::execute(const Node* node, Context& ctxt) {
         FOR_EACH(IFEXISTS)
 #undef IFEXISTS
 
-#define PARALLEL_IFEXISTS(Structure, Arity, ...)                        \
-    CASE(ParallelIfExists, Structure, Arity)                            \
+#define PARALLEL_IFEXISTS(Structure, Arity, AuxiliaryArity, ...)        \
+    CASE(ParallelIfExists, Structure, Arity, AuxiliaryArity)            \
         const auto& rel = *static_cast<RelType*>(shadow.getRelation()); \
         return evalParallelIfExists(rel, cur, shadow, ctxt);            \
     ESAC(ParallelIfExists)
@@ -1187,16 +1195,16 @@ RamDomain Engine::execute(const Node* node, Context& ctxt) {
         FOR_EACH(PARALLEL_IFEXISTS)
 #undef PARALLEL_IFEXISTS
 
-#define INDEX_IFEXISTS(Structure, Arity, ...)                 \
-    CASE(IndexIfExists, Structure, Arity)                     \
+#define INDEX_IFEXISTS(Structure, Arity, AuxiliaryArity, ...) \
+    CASE(IndexIfExists, Structure, Arity, AuxiliaryArity)     \
         return evalIndexIfExists<RelType>(cur, shadow, ctxt); \
     ESAC(IndexIfExists)
 
         FOR_EACH(INDEX_IFEXISTS)
 #undef INDEX_IFEXISTS
 
-#define PARALLEL_INDEX_IFEXISTS(Structure, Arity, ...)                  \
-    CASE(ParallelIndexIfExists, Structure, Arity)                       \
+#define PARALLEL_INDEX_IFEXISTS(Structure, Arity, AuxiliaryArity, ...)  \
+    CASE(ParallelIndexIfExists, Structure, Arity, AuxiliaryArity)       \
         const auto& rel = *static_cast<RelType*>(shadow.getRelation()); \
         return evalParallelIndexIfExists(rel, cur, shadow, ctxt);       \
     ESAC(ParallelIndexIfExists)
@@ -1223,8 +1231,8 @@ RamDomain Engine::execute(const Node* node, Context& ctxt) {
             return execute(shadow.getNestedOperation(), ctxt);
         ESAC(UnpackRecord)
 
-#define PARALLEL_AGGREGATE(Structure, Arity, ...)                       \
-    CASE(ParallelAggregate, Structure, Arity)                           \
+#define PARALLEL_AGGREGATE(Structure, Arity, AuxiliaryArity, ...)       \
+    CASE(ParallelAggregate, Structure, Arity, AuxiliaryArity)           \
         const auto& rel = *static_cast<RelType*>(shadow.getRelation()); \
         return evalParallelAggregate(rel, cur, shadow, ctxt);           \
     ESAC(ParallelAggregate)
@@ -1232,8 +1240,8 @@ RamDomain Engine::execute(const Node* node, Context& ctxt) {
         FOR_EACH(PARALLEL_AGGREGATE)
 #undef PARALLEL_AGGREGATE
 
-#define AGGREGATE(Structure, Arity, ...)                                \
-    CASE(Aggregate, Structure, Arity)                                   \
+#define AGGREGATE(Structure, Arity, AuxiliaryArity, ...)                \
+    CASE(Aggregate, Structure, Arity, AuxiliaryArity)                   \
         const auto& rel = *static_cast<RelType*>(shadow.getRelation()); \
         return evalAggregate(cur, shadow, rel.scan(), ctxt);            \
     ESAC(Aggregate)
@@ -1241,16 +1249,16 @@ RamDomain Engine::execute(const Node* node, Context& ctxt) {
         FOR_EACH(AGGREGATE)
 #undef AGGREGATE
 
-#define PARALLEL_INDEX_AGGREGATE(Structure, Arity, ...)                \
-    CASE(ParallelIndexAggregate, Structure, Arity)                     \
-        return evalParallelIndexAggregate<RelType>(cur, shadow, ctxt); \
+#define PARALLEL_INDEX_AGGREGATE(Structure, Arity, AuxiliaryArity, ...) \
+    CASE(ParallelIndexAggregate, Structure, Arity, AuxiliaryArity)      \
+        return evalParallelIndexAggregate<RelType>(cur, shadow, ctxt);  \
     ESAC(ParallelIndexAggregate)
 
         FOR_EACH(PARALLEL_INDEX_AGGREGATE)
 #undef PARALLEL_INDEX_AGGREGATE
 
-#define INDEX_AGGREGATE(Structure, Arity, ...)                 \
-    CASE(IndexAggregate, Structure, Arity)                     \
+#define INDEX_AGGREGATE(Structure, Arity, AuxiliaryArity, ...) \
+    CASE(IndexAggregate, Structure, Arity, AuxiliaryArity)     \
         return evalIndexAggregate<RelType>(cur, shadow, ctxt); \
     ESAC(IndexAggregate)
 
@@ -1283,8 +1291,8 @@ RamDomain Engine::execute(const Node* node, Context& ctxt) {
             return result;
         ESAC(Filter)
 
-#define GUARDED_INSERT(Structure, Arity, ...)                     \
-    CASE(GuardedInsert, Structure, Arity)                         \
+#define GUARDED_INSERT(Structure, Arity, AuxiliaryArity, ...)     \
+    CASE(GuardedInsert, Structure, Arity, AuxiliaryArity)         \
         auto& rel = *static_cast<RelType*>(shadow.getRelation()); \
         return evalGuardedInsert(rel, shadow, ctxt);              \
     ESAC(GuardedInsert)
@@ -1292,8 +1300,8 @@ RamDomain Engine::execute(const Node* node, Context& ctxt) {
         FOR_EACH(GUARDED_INSERT)
 #undef GUARDED_INSERT
 
-#define INSERT(Structure, Arity, ...)                             \
-    CASE(Insert, Structure, Arity)                                \
+#define INSERT(Structure, Arity, AuxiliaryArity, ...)             \
+    CASE(Insert, Structure, Arity, AuxiliaryArity)                \
         auto& rel = *static_cast<RelType*>(shadow.getRelation()); \
         return evalInsert(rel, shadow, ctxt);                     \
     ESAC(Insert)
@@ -1301,18 +1309,18 @@ RamDomain Engine::execute(const Node* node, Context& ctxt) {
         FOR_EACH(INSERT)
 #undef INSERT
 
-#define ERASE(Structure, Arity, ...)                                                 \
-    CASE(Erase, Structure, Arity)                                                    \
-        void(static_cast<RelType*>(shadow.getRelation()));                           \
-        auto& rel = *static_cast<BtreeDeleteRelation<Arity>*>(shadow.getRelation()); \
-        return evalErase(rel, shadow, ctxt);                                         \
+#define ERASE(Structure, Arity, AuxiliaryArity, ...)                                                 \
+    CASE(Erase, Structure, Arity, AuxiliaryArity)                                                    \
+        void(static_cast<RelType*>(shadow.getRelation()));                                           \
+        auto& rel = *static_cast<BtreeDeleteRelation<Arity, AuxiliaryArity>*>(shadow.getRelation()); \
+        return evalErase(rel, shadow, ctxt);                                                         \
     ESAC(Erase)
 
         FOR_EACH_BTREE_DELETE(ERASE)
 #undef ERASE
 
         CASE(SubroutineReturn)
-            for (std::size_t i = 0; i < cur.getValues().size(); ++i) {
+            for (std::size_t i = 0; i < cur.getNumValues(); ++i) {
                 if (shadow.getChild(i) == nullptr) {
                     ctxt.addReturnValue(0);
                 } else {
@@ -1342,9 +1350,11 @@ RamDomain Engine::execute(const Node* node, Context& ctxt) {
 
         CASE(Loop)
             resetIterationNumber();
+
             while (execute(shadow.getChild(), ctxt)) {
                 incIterationNumber();
             }
+
             resetIterationNumber();
             return true;
         ESAC(Loop)
@@ -1369,18 +1379,14 @@ RamDomain Engine::execute(const Node* node, Context& ctxt) {
             return execute(shadow.getChild(), ctxt);
         ESAC(DebugInfo)
 
-#define CLEAR(Structure, Arity, ...)                              \
-    CASE(Clear, Structure, Arity)                                 \
-        auto& rel = *static_cast<RelType*>(shadow.getRelation()); \
-        rel.__purge();                                            \
-        return true;                                              \
-    ESAC(Clear)
+        CASE(Clear)
+            auto* rel = shadow.getRelation();
+            rel->purge();
+            return true;
+        ESAC(Clear)
 
-        FOR_EACH(CLEAR)
-#undef CLEAR
-
-#define ESTIMATEJOINSIZE(Structure, Arity, ...)                         \
-    CASE(EstimateJoinSize, Structure, Arity)                            \
+#define ESTIMATEJOINSIZE(Structure, Arity, AuxiliaryArity, ...)         \
+    CASE(EstimateJoinSize, Structure, Arity, AuxiliaryArity)            \
         const auto& rel = *static_cast<RelType*>(shadow.getRelation()); \
         return evalEstimateJoinSize<RelType>(rel, cur, shadow, ctxt);   \
     ESAC(EstimateJoinSize)
@@ -1945,7 +1951,7 @@ bool runNested(const ram::Aggregator& aggregator) {
             default: return false;
         }
     } else if (isA<ram::UserDefinedAggregator>(aggregator)) {
-        return false;
+        return true;
     }
     return false;
 }
