@@ -18,19 +18,15 @@
 
 namespace souffle::interpreter {
 
-#define CREATE_PROVENANCE_REL(Structure, Arity, ...)                   \
-    case (Arity): {                                                    \
-        return mk<Relation<Arity, interpreter::Provenance>>(           \
-                id.getAuxiliaryArity(), id.getName(), indexSelection); \
+#define CREATE_PROVENANCE_REL(Structure, Arity, AuxiliaryArity, ...)                                       \
+    if (id.getArity() == Arity && id.getAuxiliaryArity() == AuxiliaryArity) {                              \
+        return mk<Relation<Arity, AuxiliaryArity, interpreter::Provenance>>(id.getName(), indexSelection); \
     }
 
 Own<RelationWrapper> createProvenanceRelation(
         const ram::Relation& id, const ram::analysis::IndexCluster& indexSelection) {
-    switch (id.getArity()) {
-        FOR_EACH_PROVENANCE(CREATE_PROVENANCE_REL);
-
-        default: fatal("Requested arity not yet supported. Feel free to add it.");
-    }
+    FOR_EACH_PROVENANCE(CREATE_PROVENANCE_REL);
+    fatal("Requested arity not yet supported. Feel free to add it.");
 }
 
 }  // namespace souffle::interpreter
