@@ -29,10 +29,16 @@ namespace souffle::ram {
  */
 class AutoIncrement : public Expression {
 public:
+    AutoIncrement() : Expression(NK_AutoIncrement) {}
+
     AutoIncrement* cloning() const override {
         return new AutoIncrement();
     }
 
+    static bool classof(const Node* n) {
+        return n->getKind() == NK_AutoIncrement;
+    }
+
 protected:
     void print(std::ostream& os) const override {
         os << "AUTOINC()";
