@@ -42,12 +42,16 @@ namespace souffle::ram {
 
 class Clear : public RelationStatement {
 public:
-    Clear(std::string rel) : RelationStatement(rel) {}
+    Clear(std::string rel) : RelationStatement(NK_Clear, rel) {}
 
     Clear* cloning() const override {
         return new Clear(relation);
     }
 
+    static bool classof(const Node* n) {
+        return n->getKind() == NK_Clear;
+    }
+
 protected:
     void print(std::ostream& os, int tabpos) const override {
         os << times(" ", tabpos);
