@@ -30,10 +30,16 @@ namespace souffle::ram {
  */
 class False : public Condition {
 public:
+    False() : Condition(NK_False) {}
+
     False* cloning() const override {
         return new False();
     }
 
+    static bool classof(const Node* n) {
+        return n->getKind() == NK_False;
+    }
+
 protected:
     void print(std::ostream& os) const override {
         os << "FALSE";
