@@ -19,11 +19,11 @@
 #include <cassert>
 #include <functional>
 #include <memory>
-#include <typeinfo>
 #include <utility>
 #include <vector>
 
 namespace souffle::ram {
+Node::Node(NodeKind kind) : Kind(kind) {}
 
 void Node::rewrite(const Node* oldNode, Own<Node> newNode) {
     assert(oldNode != nullptr && "old node is a null-pointer");
@@ -47,4 +47,8 @@ Node::ChildNodes Node::getChildNodes() {
     return ChildNodes(getChildren(), detail::ConstCaster());
 }
 
+Node::NodeKind Node::getKind() const {
+    return Kind;
+}
+
 }  // namespace souffle::ram
