@@ -33,7 +33,7 @@ namespace souffle::ram {
  */
 class SignedConstant : public NumericConstant {
 public:
-    explicit SignedConstant(RamDomain val) : NumericConstant(val) {}
+    explicit SignedConstant(RamDomain val) : NumericConstant(NK_SignedConstant, val) {}
 
     /** @brief Get value of the constant. */
     RamDomain getValue() const {
@@ -45,6 +45,10 @@ public:
         return new SignedConstant(getValue());
     }
 
+    static bool classof(const Node* n) {
+        return n->getKind() == NK_SignedConstant;
+    }
+
 protected:
     void print(std::ostream& os) const override {
         os << "NUMBER(" << constant << ")";
