@@ -31,7 +31,7 @@ namespace souffle::ram {
  */
 class StringConstant : public Expression {
 public:
-    StringConstant(std::string constant) : constant(constant) {}
+    StringConstant(std::string constant) : Expression(NK_StringConstant), constant(constant) {}
 
     /** @brief Get constant */
     const std::string& getConstant() const {
@@ -42,6 +42,10 @@ public:
         return new StringConstant(constant);
     }
 
+    static bool classof(const Node* n) {
+        return n->getKind() == NK_StringConstant;
+    }
+
 protected:
     void print(std::ostream& os) const override {
         os << "STRING(\"" << stringify(constant) << "\")";
