@@ -38,12 +38,16 @@ namespace souffle::ram {
  */
 class Swap : public BinRelationStatement {
 public:
-    Swap(std::string f, std::string s) : BinRelationStatement(f, s) {}
+    Swap(std::string f, std::string s) : BinRelationStatement(NK_Swap, f, s) {}
 
     Swap* cloning() const override {
         return new Swap(first, second);
     }
 
+    static bool classof(const Node* n) {
+        return n->getKind() == NK_Swap;
+    }
+
 protected:
     void print(std::ostream& os, int tabpos) const override {
         os << times(" ", tabpos);
