@@ -30,10 +30,16 @@ namespace souffle::ram {
  */
 class True : public Condition {
 public:
+    True() : Condition(NK_True) {}
+
     True* cloning() const override {
         return new True();
     }
 
+    static bool classof(const Node* n) {
+        return n->getKind() == NK_True;
+    }
+
 protected:
     void print(std::ostream& os) const override {
         os << "TRUE";
